import "./skills.js"
import { skillsByEvent, skillsById, tickUpgrades, events } from "./skills-addon-2.js"
import { openSkillsMenu } from "./ui.js"
import { world } from "mojang-minecraft"

function setup(player) {
    player.addTag("sa2:setup")
    for (const skill of Object.values(skillsById)) {
        player.runCommand(`scoreboard objectives add ${skill.id}X dummy "${skill.displayName} XP"`)
        player.runCommand(`scoreboard players set @s ${skill.id}X 0`)
        player.runCommand(`scoreboard objectives add ${skill.id}L dummy "${skill.displayName}"`)
        player.runCommand(`scoreboard players set @s ${skill.id}L 0`)
    }
}

function skillTableSneak(player) {
    player.runCommand(`tellraw @s {"rawtext": [{"text": "§lSkills Table"}]}`)
    for (const skill of Object.values(skillsById)) {
        let upgrades = []
        for (const tag of player.getTags()) {
            if (tag.startsWith(skill.id)) {
                upgrades.push(skill.upgradesById[tag].displayName)
            }
        }

        if (upgrades.length == 0) { upgrades = ["§cNo upgrades!"] }

        player.runCommand(`tellraw @s {"rawtext": [
            {"text": "\n§l${skill.displayName}§r - §lXP:§r "},
            {"score":{"name": "@s","objective": "${skill.id}X"}},
            {"text": "/750  §lSkill Points:§r "},
            {"score":{"name": "@s","objective": "${skill.id}L"}},
            {"text": "  §lUpgrades:§r ${upgrades.join(', ')}\n"}
            ]}`)
    }
}

world.events.tick.subscribe((event) => {
    for (const player of world.getPlayers()) {
        if (!player.hasTag("sa2:setup")) {
            setup(player)
        }
        if (player.hasTag("sa2:openSkillsMenu")) {
            player.removeTag("sa2:openSkillsMenu")
            openSkillsMenu(player)
        }
        if (player.hasTag("sa2:skillTableSneak")) {
            player.removeTag("sa2:skillTableSneak")
            skillTableSneak(player)
        }

        for (const tag of player.getTags()) {
            if (Object.keys(tickUpgrades).includes(tag)) {
                tickUpgrades[tag]({
                    player: player,
                    tick: event
                })
            }
        }
    }
})

world.events.blockBreak.subscribe((event) => {
    for (const skill of skillsByEvent[events.blockBreak]) {
        skill.callback(skill, event)
    }
})

world.events.entityHit.subscribe((event) => {
    for (const skill of skillsByEvent[events.entityHit]) {
        skill.callback(skill, event)
    }
})