export var tickUpgrades = {}
export var skillsById = {}
export var skillsByEvent = {
    0: [],
    1: []
}

export const events = {
    blockBreak: 0,
    entityHit: 1,
}

function addSkill(event, skill) {
    skillsById[skill.id] = skill
    skillsByEvent[event].push(skill)
}

function getUpgradesById(upgrades) {
    let obj = {}
    for (const upgrade of upgrades) {
        obj[upgrade.id] = upgrade
    }

    return obj
}

export function activateSkill(skill, player, event) {
    // add xp
    player.runCommand(`scoreboard players add @s ${skill.id}X 1`)
    try { player.runCommand(`tag @s[scores={${skill.id}X=750..}] add sa2:leveledup`) } catch { }
    if (player.hasTag("sa2:leveledup")) {
        player.removeTag("sa2:leveledup")
        player.runCommand("playsound random.levelup @s")
        player.runCommand(`scoreboard players add @s ${skill.id}L 1`)
        player.runCommand(`scoreboard players set @s ${skill.id}X 0`)
        player.runCommand(`tellraw @s {"rawtext": [
        {"text": "You just earned a §l${skill.displayName}§r skill point! You now have "},
        {"score": {"name": "@s", "objective": "${skill.id}L"}}, {"text": "."}]}`)
    }

    // call upgrades
    for (const upgrade of skill.upgrades) {
        if (player.hasTag(upgrade.id) && !upgrade.runOnTick) {
            upgrade.callback(event)
        }
    }
}

export function newSkill(id, displayName, event, upgrades, callback) { // id must be less than 16 characters
    if (id.length > 16) {
        throw "Skill id cannot be longer than 16 characters!"
    }

    addSkill(event, {
        id: id,
        displayName: displayName,
        upgrades: upgrades,
        upgradesById: getUpgradesById(upgrades),
        callback: callback
    })
}

export function newUpgradeGroup(skillId, upgrades) {
    for (let x = 0; x < upgrades.length; x++) {
        upgrades[x] = `${skillId}:${upgrades[x]}`
    }

    return upgrades
}

export function newUpgrade(id, displayName, skillId, price, callback, upgradeGroup = [], runOnTick = false) {
    let newId = `${skillId}:${id}`

    if (runOnTick) {
        tickUpgrades[newId] = callback
    }

    return {
        id: newId,
        displayName: displayName,
        skillId: skillId,
        price: price,
        upgradeGroup: upgradeGroup,
        callback: callback,
        runOnTick: runOnTick
    }
}

