import * as skills from "./skills-addon-2.js"
import { MinecraftEffectTypes, BlockProperties } from "mojang-minecraft"

function getRandomInt(max, min = 0) {
    return Math.max(Math.floor(Math.random() * max), min)
}

// mining

const ores = [
    "minecraft:deepslate_copper_ore",
    "minecraft:deepslate_coal_ore",
    "minecraft:deepslate_emerald_ore",
    "minecraft:deepslate_redstone_ore",
    "minecraft:lit_deepslate_redstone_ore",
    "minecraft:emerald_ore",
    "minecraft:quartz_ore",
    "minecraft:nether_gold_ore",
    "minecraft:deepslate_iron_ore",
    "minecraft:deepslate_gold_ore",
    "minecraft:deepslate_diamond_ore",
    "minecraft:deepslate_lapis_ore",
    "minecraft:copper_ore",
    "minecraft:coal_ore",
    "minecraft:redstone_ore",
    "minecraft:lit_redstone_ore",
    "minecraft:lapis_ore",
    "minecraft:diamond_ore",
    "minecraft:gold_ore",
    "minecraft:iron_ore",
    "minecraft:ancient_debris"
]

const miningSkillBlocks = [
    "minecraft:deepslate",
    "minecraft:stone",
    "minecraft:cobblestone",
    "minecraft:mossy_cobblestone",
    "minecraft:cobbled_deepslate",
    "minecraft:obsidian",
    "minecraft:basalt",
    "minecraft:blackstone"
]

function experience(player, blockId, blockPos, level) {
    switch (blockId) {
        case "minecraft:coal_ore":
        case "minecraft:deepslate_coal_ore":
        case "minecraft:nether_gold_ore":
            player.runCommand(`summon minecraft:xp_orb ${blockPos.x} ${blockPos.y} ${blockPos.z}`)
            player.runCommand(`summon minecraft:xp_orb ${blockPos.x} ${blockPos.y} ${blockPos.z}`)
            break
        case "minecraft:diamond_ore":
        case "minecraft:deepslate_diamond_ore":
        case "minecraft:emerald_ore":
        case "minecraft:deepslate_emerald_ore":
            for (let x = 0; x < getRandomInt(7, 2); x++) {
                player.runCommand(`summon minecraft:xp_orb ${blockPos.x} ${blockPos.y} ${blockPos.z}`)
            }
            break
        case "minecraft:lapis_ore":
        case "minecraft:deepslate_lapis_ore":
        case "minecraft:quartz_ore":
        case "minecraft:redstone_ore":
            for (let x = 0; x < getRandomInt(5, 2); x++) {
                player.runCommand(`summon minecraft:xp_orb ${blockPos.x} ${blockPos.y} ${blockPos.z}`)
            }
            break
        default:
            if ((level > 1 && ores.includes(blockId)) || (level == 3 && getRandomInt(10) == 0)) {
                player.runCommand(`summon minecraft:xp_orb ${blockPos.x} ${blockPos.y} ${blockPos.z}`)
                player.runCommand(`summon minecraft:xp_orb ${blockPos.x} ${blockPos.y} ${blockPos.z}`)
            }
    }
}

const hasteUpgradeGroup = skills.newUpgradeGroup("sa2mining", ["haste1", "haste2", "haste3"])
const spelunkerUpgradeGroup = skills.newUpgradeGroup("sa2mining", ["spelunker1", "spelunker2", "spelunker3"])
const xpUpgradeGroup = skills.newUpgradeGroup("sa2mining", ["xp1", "xp2", "xp3"])

skills.newSkill("sa2mining", "§8Mining", skills.events.blockBreak,
    [
        skills.newUpgrade("haste1", "Haste I", "sa2mining", 1, (event) => {
            event.player.addEffect(MinecraftEffectTypes.haste, 100, 1)
        }, hasteUpgradeGroup),
        skills.newUpgrade("haste2", "Haste II", "sa2mining", 3, (event) => {
            event.player.addEffect(MinecraftEffectTypes.haste, 100, 2)
        }, hasteUpgradeGroup),
        skills.newUpgrade("haste3", "Haste III", "sa2mining", 5, (event) => {
            event.player.addEffect(MinecraftEffectTypes.haste, 100, 3)
        }, hasteUpgradeGroup),


        skills.newUpgrade("spelunker1", "Spelunker I", "sa2mining", 3, (event) => {
            if (getRandomInt(10) == 0) {
                let blockPos = event.block.location
                event.player.runCommand(`summon sa2:loot ${blockPos.x} ${blockPos.y} ${blockPos.z} sa2mining:spelunker1`)
            }
        }, spelunkerUpgradeGroup),
        skills.newUpgrade("spelunker2", "Spelunker II", "sa2mining", 6, (event) => {
            if (getRandomInt(10) == 0) {
                let blockPos = event.block.location
                event.player.runCommand(`summon sa2:loot ${blockPos.x} ${blockPos.y} ${blockPos.z} sa2mining:spelunker2`)
            }
        }, spelunkerUpgradeGroup),
        skills.newUpgrade("spelunker3", "Spelunker III", "sa2mining", 9, (event) => {
            if (getRandomInt(10) == 0) {
                let blockPos = event.block.location
                event.player.runCommand(`summon sa2:loot ${blockPos.x} ${blockPos.y} ${blockPos.z} sa2mining:spelunker3`)
            }
        }, spelunkerUpgradeGroup),

        skills.newUpgrade("xp1", "Experience I", "sa2mining", 2, (event) => {
            experience(event.player, event.brokenBlockPermutation.type.id, event.block.location, 1)
        }, xpUpgradeGroup),
        skills.newUpgrade("xp2", "Experience II", "sa2mining", 4, (event) => {
            experience(event.player, event.brokenBlockPermutation.type.id, event.block.location, 2)
        }, xpUpgradeGroup),
        skills.newUpgrade("xp3", "Experience III", "sa2mining", 6, (event) => {
            experience(event.player, event.brokenBlockPermutation.type.id, event.block.location, 3)
        }, xpUpgradeGroup)
    ],

    (self, event) => {
        let blockId = event.brokenBlockPermutation.type.id
        if (miningSkillBlocks.includes(blockId) || ores.includes(blockId)) {
            skills.activateSkill(self, event.player, event)
        }
    })

// lumber

const lumberSkillBlocks = [
    "minecraft:log",
    "minecraft:log2"
]

function foraging_fortune(possibleItems, chance, player, oldLogType, newLogType, blockPos) {
    let woodType
    if (oldLogType) {
        woodType = oldLogType.value
    } else {
        woodType = newLogType.value
    }

    if (getRandomInt(chance) == 0) {
        switch (getRandomInt(possibleItems)) {
            case 0: player.runCommand(`summon sa2:loot ${blockPos.x} ${blockPos.y} ${blockPos.z} sa2:${woodType}_sapling`); break;
            case 1: player.runCommand(`summon sa2:loot ${blockPos.x} ${blockPos.y} ${blockPos.z} sa2:stick`); break;

            case 2: player.runCommand(`summon sa2:loot ${blockPos.x} ${blockPos.y} ${blockPos.z} sa2:${woodType}_planks`); break;
        }
    }
}

const fortuneUpgradeGroup = skills.newUpgradeGroup("sa2lumber", ["fortune1", "fortune2", "fortune3", "fortune4", "fortune5"])

skills.newSkill("sa2lumber", "§aLumber", skills.events.blockBreak,
    [
        skills.newUpgrade("fortune1", "Fortune I", "sa2lumber", 1, (event) => {
            let blockPerm = event.brokenBlockPermutation
            foraging_fortune(2, 8, event.player, blockPerm.getProperty(BlockProperties.oldLogType), blockPerm.getProperty(BlockProperties.newLogType), event.block.location)
        }, fortuneUpgradeGroup),
        skills.newUpgrade("fortune2", "Fortune II", "sa2lumber", 1, (event) => {
            let blockPerm = event.brokenBlockPermutation
            foraging_fortune(3, 8, event.player, blockPerm.getProperty(BlockProperties.oldLogType), blockPerm.getProperty(BlockProperties.newLogType), event.block.location)
        }, fortuneUpgradeGroup),
        skills.newUpgrade("fortune3", "Fortune III", "sa2lumber", 2, (event) => {
            let blockPerm = event.brokenBlockPermutation
            foraging_fortune(3, 6, event.player, blockPerm.getProperty(BlockProperties.oldLogType), blockPerm.getProperty(BlockProperties.newLogType), event.block.location)
        }, fortuneUpgradeGroup),
        skills.newUpgrade("fortune4", "Fortune IV", "sa2lumber", 2, (event) => {
            let blockPerm = event.brokenBlockPermutation
            foraging_fortune(3, 4, event.player, blockPerm.getProperty(BlockProperties.oldLogType), blockPerm.getProperty(BlockProperties.newLogType), event.block.location)
        }, fortuneUpgradeGroup),
        skills.newUpgrade("fortune5", "Fortune V", "sa2lumber", 3, (event) => {
            let blockPerm = event.brokenBlockPermutation
            foraging_fortune(3, 2, event.player, blockPerm.getProperty(BlockProperties.oldLogType), blockPerm.getProperty(BlockProperties.newLogType), event.block.location)
        }, fortuneUpgradeGroup)
    ],

    (self, event) => {
        if (lumberSkillBlocks.includes(event.brokenBlockPermutation.type.id)) {
            skills.activateSkill(self, event.player, event)
        }
    })

// combat

const lightweightUpgradeGroup = skills.newUpgradeGroup("sa2combat", ["lightweight1", "lightweight2"])
const vexTamerUpgradeGroup = skills.newUpgradeGroup("sa2combat", ["vextamer1", "vextamer2", "vextamer3"])

skills.newSkill("sa2combat", "§cCombat", skills.events.entityHit,
    [
        skills.newUpgrade("lightweight1", "Lightweight I", "sa2combat", 1, (event) => {
            if (getRandomInt(4) == 0) { event.entity.addEffect(MinecraftEffectTypes.speed, 200, 1) }
        }, lightweightUpgradeGroup),
        skills.newUpgrade("lightweight2", "Lightweight II", "sa2combat", 1, (event) => {
            event.entity.addEffect(MinecraftEffectTypes.speed, 200, 1)
        }, lightweightUpgradeGroup),


        skills.newUpgrade("paralyze", "Paralyze", "sa2combat", 3, (event) => {
            if (getRandomInt(4) == 0) { event.hitEntity.addEffect(MinecraftEffectTypes.slowness, 100, 2) }
        }),


        skills.newUpgrade("toxicblade", "Toxic Blade", "sa2combat", 5, (event) => {
            if (getRandomInt(8) == 0) { event.hitEntity.addEffect(MinecraftEffectTypes.wither, 60, 3) }
        }),


        skills.newUpgrade("sharppoint", "Sharp Point", "sa2combat", 5, (event) => {
            if (getRandomInt(8) == 0) { event.entity.addEffect(MinecraftEffectTypes.strength, 200, 1) }
        }),


        skills.newUpgrade("vextamer1", "Vex Tamer I", "sa2combat", 5, (event) => {
            if (getRandomInt(8) == 0) { event.hitEntity.runCommand("summon sa2:friendly_vex ~~~") }
        }, vexTamerUpgradeGroup),

        skills.newUpgrade("vextamer2", "Vex Tamer II", "sa2combat", 6, (event) => {
            if (getRandomInt(6) == 0) { event.hitEntity.runCommand("summon sa2:friendly_vex ~~~") }
        }, vexTamerUpgradeGroup),

        skills.newUpgrade("vextamer3", "Vex Tamer III", "sa2combat", 7, (event) => {
            if (getRandomInt(4) == 0) { event.hitEntity.runCommand("summon sa2:friendly_vex ~~~") }
        }, vexTamerUpgradeGroup)
    ],

    (self, event) => {
        if (event.hitEntity && event.entity.id == "minecraft:player") {
            skills.activateSkill(self, event.entity, event)
        }
    })

// farming

const farmingSkillPlants = [
    "minecraft:beetroot",
    "minecraft:wheat",
    "minecraft:carrots",
    "minecraft:potatoes"
]

const farmingSkillBlocks = [
    "minecraft:melon_block",
    "minecraft:pumpkin"
]

const delicateTouchUpgradeGroup = skills.newUpgradeGroup("sa2farming", ["delicatetouch1", "delicatetouch2"])

skills.newSkill("sa2farming", "§eFarming", skills.events.blockBreak,
    [
        skills.newUpgrade("delicatetouch1", "Delicate Touch I", "sa2farming", 2, (event) => {
            if (getRandomInt(4) == 0) {
                let blockId = event.brokenBlockPermutation.type.id
                let blockPos = event.block.location
                if (farmingSkillPlants.includes(blockId)) {
                    event.player.runCommand(`setblock ${blockPos.x} ${blockPos.y} ${blockPos.z} ${blockId} 7`)
                } else {
                    event.player.runCommand(`setblock ${blockPos.x} ${blockPos.y} ${blockPos.z} ${blockId}`)
                }
                event.player.runCommand(`particle minecraft:crop_growth_emitter ${blockPos.x} ${blockPos.y} ${blockPos.z}`)
            }
        }, delicateTouchUpgradeGroup),
        skills.newUpgrade("delicatetouch2", "Delicate Touch II", "sa2farming", 6, (event) => {
            if (getRandomInt(2) == 0) {
                let blockId = event.brokenBlockPermutation.type.id
                let blockPos = event.block.location
                if (farmingSkillPlants.includes(blockId)) {
                    event.player.runCommand(`setblock ${blockPos.x} ${blockPos.y} ${blockPos.z} ${blockId} 7`)
                } else {
                    event.player.runCommand(`setblock ${blockPos.x} ${blockPos.y} ${blockPos.z} ${blockId}`)
                }
                event.player.runCommand(`particle minecraft:crop_growth_emitter ${blockPos.x} ${blockPos.y} ${blockPos.z}`)
            }
        }, delicateTouchUpgradeGroup)
    ],

    (self, event) => {
        let blockId = event.brokenBlockPermutation.type.id
        if (farmingSkillBlocks.includes(blockId) || (farmingSkillPlants.includes(blockId) && event.brokenBlockPermutation.getProperty(BlockProperties.growth).value == 7)) {
            skills.activateSkill(self, event.player, event)
        }
    })

// defense

const cmedicUpgradeGroup = skills.newUpgradeGroup("sa2defense", ["cmedic1", "cmedic2", "cmedic3"])
const frenzyUpgradeGroup = skills.newUpgradeGroup("sa2defense", ["frenzy1", "frenzy2"])

skills.newSkill("sa2defense", "§bDefense", skills.events.entityHit,
    [
        skills.newUpgrade("frenzy1", "Frenzy I", "sa2defense", 1, (event) => {
            if (event.player.getComponent("minecraft:health").current < 5) {
                event.player.addEffect(MinecraftEffectTypes.resistance, 20, 1)
                event.player.addEffect(MinecraftEffectTypes.strength, 20, 1)
            }
        }, frenzyUpgradeGroup, true), // called every tick, passed event is skills tick event (see main.js)

        skills.newUpgrade("frenzy2", "Frenzy II", "sa2defense", 3, (event) => {
            if (event.player.getComponent("minecraft:health").current < 10) {
                event.player.addEffect(MinecraftEffectTypes.resistance, 20, 1)
                event.player.addEffect(MinecraftEffectTypes.strength, 20, 1)
            }
        }, frenzyUpgradeGroup, true),


        skills.newUpgrade("cmedic1", "Combat Medic I", "sa2defense", 2, (event) => {
            if (getRandomInt(4) == 0) {
                let healthComponent = event.hitEntity.getComponent("minecraft:health")

                // add 50% of health lost rounded to the nearest int to current health
                healthComponent.setCurrent(healthComponent.current + Math.round(0.5 * (healthComponent.value - healthComponent.current)))
            }
        }, cmedicUpgradeGroup),
        skills.newUpgrade("cmedic2", "Combat Medic II", "sa2defense", 4, (event) => {
            if (getRandomInt(4) == 0) {
                let healthComponent = event.hitEntity.getComponent("minecraft:health")

                // add 75% of health lost rounded to the nearest int to current health
                healthComponent.setCurrent(healthComponent.current + Math.round(0.75 * (healthComponent.value - healthComponent.current)))
            }
        }, cmedicUpgradeGroup),
        skills.newUpgrade("cmedic3", "Combat Medic III", "sa2defense", 6, (event) => {
            if (getRandomInt(4) == 0) {
                // give 100% of health back
                event.hitEntity.getComponent("minecraft:health").resetToMaxValue()
            }
        }, cmedicUpgradeGroup)
    ],

    (self, event) => {
        if (event.hitEntity && event.hitEntity.id == "minecraft:player") {
            skills.activateSkill(self, event.hitEntity, event)
        }
    })